@@ -9,6 +9,8 @@ var bkgdColor = "#000000";
 var btnSize = new size(134,134);
 var btnIconSize = new size(48,48);
 
+var START_TIMEOUT = 1500
+
 //images set by the theme
 var btnBkgdNormal;
 var btnBkgdHover;
@@ -109,7 +111,7 @@ function buttonEvent(domObj, event, index)
 	domObj.style.color = btnTextColorSelected;
 	domObj.style.fontSize =  btnTextSizeSelected;
 	appLaunching = true;
-	setTimeout (appLaunchTimeout, 3000, domObj); //tbd: get notification
+	setTimeout (appLaunchTimeout, START_TIMEOUT, domObj); //tbd: get notification
 	document.body.style.cursor='wait';
 	launchAppFromIndex(index);
     }
