@@ -11,7 +11,8 @@
  * Contributors:
  * Michael Frey <michael.frey@pepper.com>
  * Rusty Lynch <rusty.lynch@intel.com>
- * 
+ * Horace Li <horace.li@intel.com>
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
@@ -42,16 +43,19 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glib-object.h>
+#include <libxml/parser.h>
 
-#include <libhildondesktop/libhildondesktop.h>
 #include "mobile-basic-home-plugin.h"
 #include "gtkmozembed.h"
 
+#include <libhildondesktop/libhildondesktop.h>
 #include <libhildonwm/hd-wm.h>
 #include <libosso.h>
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-note.h>
 
+#define GMENU_I_KNOW_THIS_IS_UNSTABLE
+#include <gmenu-tree.h>
 
 #define FILE_URL_PREFIX "file://"
 #define PROFILE_DIR "/usr/share/mobile-basic-flash"
@@ -62,14 +66,16 @@
 #define DESKTOP_FILE_SUFFIX ".desktop"
 #define DESKTOP_GROUP "Desktop Entry"
 
-#define DESKTOP_DIR    "/usr/share/mobile-basic-flash/applications"
-#define CP_PLUGINS_DIR "/usr/share/applications/hildon-control-panel"
+#define DESKTOP_DIR    "/usr/share/applications"
+
+#define APPLICATIONS_MENU "/etc/xdg/menus/applications.menu"
+#define PREFERENCES_MENU "/etc/xdg/menus/preferences.menu"
 
 //TBD: use theme pattern to get fonts
-#define ICON_DIR    "/usr/share/mobile-basic-flash/icons"
+#define PIXMAP_DIR    "/usr/share/pixmaps"
 
 //TBD: use gconf header to replace all gconf defines
-#define BKGD_PATH             "/desktop/gnome/background"
+#define BKGD_PATH             "/desktop/moblin/background"
 #define BKGD_FILE             BKGD_PATH "/picture_filename"
 #define BKGD_PIC_OPTIONS      BKGD_PATH "/picture_options"
 #define BKGD_PRIMARY_COLOR    BKGD_PATH "/primary_color"
@@ -84,9 +90,14 @@
 #define MARQUEE_KEY     HILDON_DESKTOP_GCONF_PATH "/marquee/hide"
 #define HOMEPAGE_KEY    HILDON_DESKTOP_GCONF_PATH "/htmlhomeplugin/homepage"
 #define FLASHMOVIE_KEY  HILDON_DESKTOP_GCONF_PATH "/htmlhomeplugin/flashmoviename"
+#define ONLYSHOWIN_FILTER_KEY  HILDON_DESKTOP_GCONF_PATH "/htmlhomeplugin/onlyshowin_filter"
+#define ONLYSHOWIN_IGNORE_KEY  HILDON_DESKTOP_GCONF_PATH "/htmlhomeplugin/onlyshowin_ignore"
 
 #define MARQUEE_GCONF_PATH "/apps/marquee-plugins"
-#define ACTIVE_CAT_KEY     MARQUEE_GCONF_PATH "/active"
+#define ACTIVE_CAT_KEY MARQUEE_GCONF_PATH "/active"
+#define CATEGORIES_KEY MARQUEE_GCONF_PATH "/categories"
+
+#define UNKNOWN_ICON_NAME     "unknown"
 
 #define DialogDestroy 111
 
@@ -97,21 +108,6 @@ HD_DEFINE_PLUGIN (MobileBasicHomePlugin, mobile_basic_home_plugin, HILDON_DESKTO
 
 static GtkWidget* launch_banner = NULL;
 
-//#define LOG_FILE
-#ifdef LOG_FILE
-#define DBG_LOG trace
-static void trace (const gchar * msg) {
-  FILE *fp;
-  if (fp = fopen("/tmp/mobile_trace.log", "a")) {
-	  fprintf (fp, "%s\n", msg);
-	  fclose (fp);
-  }
-}
-#else
-#define DBG_LOG 
-#endif
-
-
 typedef struct {
 	int index;
 	gchar *plugin;
@@ -127,52 +123,78 @@ typedef struct {
 struct _plugin_context_t {
         GtkWidget *container;
         GList *app_list;
+        GHashTable *cat_table;
         osso_context_t *osso_context;
         GConfClient *client;
         const char *active;
 };
 
-
+extern void init_window_helper (plugin_context_t *context);
 static GdkFilterReturn mobile_basic_home_x_event_handler (GdkXEvent *xevent, GdkEvent *event, gpointer data);
 static void home_screen_changed (HildonDesktopHomeItem *item);
 static void start_plugin(plugin_context_t *, gchar*);
 
+void hide_banner(void);
+void net_stop_cb (GtkMozEmbed *embed, gpointer data);
+void js_status_cb (GtkMozEmbed *embed, gpointer data);
+gint compare_items(gconstpointer a, gconstpointer b);
+static gboolean is_app_in_ignore_list (GSList *ignore_list, gchar *app);
+
+// escape " and '
+static gchar *strescape_quotes (gchar *src) {
+    int len = strlen(src);
+    gchar *tmp = malloc ((len*2)+1);
+    int j=0;
+    for (int i=0; i<len; i++,j++) {
+        if (src[i]=='\'' || src[i]=='"') {
+            tmp[j++]='\\';
+        }            
+        tmp[j]=src[i];
+    }
+    tmp[j]='\0';
+    return tmp;
+}
+
 static void update_background(plugin_context_t *context) {
 	gchar *pic, *opt, *color;
 
-	DBG_LOG ("update_background");
-
 	pic = gconf_client_get_string (context->client,BKGD_FILE,NULL);
 	opt = gconf_client_get_string (context->client,BKGD_PIC_OPTIONS,NULL);
 	color = gconf_client_get_string (context->client,BKGD_PRIMARY_COLOR,NULL);
 
 	//if a picture is selected (not "no Wallpaper"), make sure it is valid
 	if ((pic != NULL) &&
-	    !g_utf8_validate(pic, -1, NULL) || !g_file_test(pic, G_FILE_TEST_EXISTS)) {
+	    (!g_utf8_validate(pic, -1, NULL) || !g_file_test(pic, G_FILE_TEST_EXISTS))) {
 		pic = g_filename_from_utf8 (pic, -1, NULL,NULL, NULL);
 	}
 
+        //escape quotes
+        if (pic) {
+            gchar *tmp = pic;
+            pic = strescape_quotes (tmp);
+            g_free (tmp);
+        }
+
 	gchar *url = g_strdup_printf("javascript:setBackground(['%s', '%s', '%s'])", 
 				     pic, opt, color);
+
 	gtk_moz_embed_load_url(GTK_MOZ_EMBED(context->container), url);
 	g_free(url);
 
 	if (pic) {
-		g_free (pic);
+            g_free (pic);
 	}
        	if (opt) {
-		g_free (opt);
+            g_free (opt);
 	}
 	if (color) {
-		g_free (color);
+            g_free (color);
 	}
 }
 
 
 static void update_theme(plugin_context_t *context) {
 
-	DBG_LOG ("interface_changed");
-
 	gchar *theme, *icon_theme, *font;
 	theme =      gconf_client_get_string (context->client, GTK_THEME, NULL);
 	icon_theme = gconf_client_get_string (context->client, ICON_THEME, NULL);
@@ -222,31 +244,16 @@ static void add_application(gpointer data, gpointer payload)
 {
 	application_entry_t *item = (application_entry_t *)data;
 	plugin_context_t *c = (plugin_context_t *)payload;
-	char **catv = g_strsplit(item->cat, ";", 0);
 
-	/*
-	 * Handle control panel plugins as a special case, where we
-	 * only want to add buttons if the category is "Settings"
-	 */
-	if (item->plugin) {
-		if (!g_ascii_strcasecmp("Settings", c->active))
+	if (!g_ascii_strcasecmp("All", c->active)) {
+		if (g_ascii_strcasecmp(item->cat, "Preferences"))
 			__add_application(c, item);
-		return;
 	}
-
-	/* 
-	 * If any of the ';' delimited entries from the desktop 
-	 * 'Categories' of the given app match the currently active
-	 * category, then add the app to the content.
-	 */
-	for ( int i=0; catv[i]; i++ ) {
-		if (!g_ascii_strcasecmp("All", c->active) ||
-		    !g_ascii_strcasecmp(catv[i], c->active)) {
+	else {
+		if (!g_ascii_strcasecmp(item->cat, c->active))
 			__add_application(c, item);
-			return;
-		}
 	}
-
+	return;
 }
 
 static gboolean init_program(plugin_context_t *c)
@@ -262,9 +269,8 @@ static gboolean init_program(plugin_context_t *c)
 	return FALSE;
 }
 
-static void hide_banner ()
+void hide_banner (void)
 {
-	DBG_LOG ("hide_banner");
 	if (launch_banner != NULL) {
 		gtk_widget_destroy(launch_banner);
                 launch_banner = NULL;
@@ -315,7 +321,7 @@ static HDWMEntryInfo* find_running_app(application_entry_t *item)
   applications_list = hd_wm_get_applications (hdwm);
 
   guint len = g_list_length(applications_list);
-  g_debug("find_running_app #running apps=%i\n", len);
+  printf ("find_running_app #running apps=%i\n", len);
 
   for (int i = 0; i < len; i++) {
 
@@ -325,7 +331,7 @@ static HDWMEntryInfo* find_running_app(application_entry_t *item)
     gchar *exec_name = parse_exec(hd_wm_application_get_exec(app));
     const gchar *class_name = hd_wm_application_get_class_name(app);
 
-    g_debug("searching appName=%s execName=%s className=%s appInfo=%x\n", 
+    g_debug("searching appName=%s execName=%s className=%s appInfo=%p\n", 
 	    app_name, exec_name, class_name, 
 	    HD_WM_ENTRY_INFO(hd_wm_application_get_active_window(app)));
     
@@ -337,7 +343,7 @@ static HDWMEntryInfo* find_running_app(application_entry_t *item)
         {
 
         app_info = HD_WM_ENTRY_INFO(hd_wm_application_get_active_window(app));
-        g_debug("found already running returning app_name=%s app_exec=%s app_info=%x\n", 
+        g_debug("found already running returning app_name=%s app_exec=%s app_info=%p\n", 
 		item->name, item_exec_name, app_info);
 
         g_free(exec_name);
@@ -466,6 +472,9 @@ static void launch_app(plugin_context_t *c, application_entry_t *item)
 			close (fd);
 		}
 	}
+
+	// Hide banner after 15s if not already hidden
+	//g_timeout_add (15*1000, force_banner_hide, NULL);
 }
 
 static void start_app_from_index(plugin_context_t *c, int index) 
@@ -481,12 +490,9 @@ static void start_app_from_index(plugin_context_t *c, int index)
 		g_list_foreach(c->app_list, dump_application, c->container);
 		return;
 	}
+
 	item = (application_entry_t *)l->data;
 
-	gchar* msg = g_strdup_printf("Starting %s...", item->name);
-
- 	item = (application_entry_t *)l->data;
-
 	launch_app(c, item);
 }
 
@@ -508,8 +514,6 @@ static void start_app_from_path(plugin_context_t *c, gchar *path)
 //start an application based on it's name as defined in the .desktop file
 static void start_app_from_name(plugin_context_t *c, gchar* appName) 
 {
-        gchar *program = NULL;
-	GError *error = NULL;
 	application_entry_t *item = NULL;
 
 	guint len = g_list_length(c->app_list);
@@ -536,8 +540,6 @@ static void start_app_from_name(plugin_context_t *c, gchar* appName)
 //start an application based on it's ID as defined in the .desktop file
 static void start_app_from_id(plugin_context_t *c, char* appId) 
 {
-        gchar *program = NULL;
-	GError *error = NULL;
 	application_entry_t *item = NULL;
 
 	guint len = g_list_length(c->app_list);
@@ -656,180 +658,301 @@ get_localized_name(GKeyFile *key_file, const char *group)
 gchar *lang, *namefield, *name = NULL;
 const gchar *mylang;
 
-	/* retrieve the language from the env var */
-	if((mylang = g_getenv("LANG")) != NULL) {
+        /* retrieve the language from the env var */
+        if((mylang = g_getenv("LANG")) != NULL) {
 
-		lang = g_strdup(mylang);
+                lang = g_strdup(mylang);
 
-		/* if the value has .UTF-8 appended, we */
-		/* have to remove that as it's unneeded */
-		g_strdelimit(lang, ".", '\0');
+                /* if the value has .UTF-8 appended, we */
+                /* have to remove that as it's unneeded */
+                g_strdelimit(lang, ".", '\0');
 
-		/* this is the nametag for our language */
-		namefield = g_strdup_printf("%s[%s]", 
-			DESKTOP_ENTRY_NAME_FIELD, lang);
+                /* this is the nametag for our language */
+                namefield = g_strdup_printf("%s[%s]",
+                        DESKTOP_ENTRY_NAME_FIELD, lang);
 
-		/* try to get out Name value */
-		name = g_key_file_get_string(key_file, group,
-			namefield, NULL);
+                /* try to get out Name value */
+                name = g_key_file_get_string(key_file, group,
+                        namefield, NULL);
 
-		g_free(lang);
-		g_free(namefield);
-	}
+                g_free(lang);
+                g_free(namefield);
+        }
 
-	/* if we found no localized name, get the default */
-	if(name == NULL) {
-		name = g_key_file_get_string(key_file, 
-			group, DESKTOP_ENTRY_NAME_FIELD, 
-			NULL);
-	}
+        /* if we found no localized name, get the default */
+        if(name == NULL) {
+                name = g_key_file_get_string(key_file,
+                        group, DESKTOP_ENTRY_NAME_FIELD,
+                        NULL);
+        }
 
-	return name;
+        return name;
+}
+
+static gboolean is_app_in_ignore_list (GSList *ignore_list, gchar *appconf)
+{
+  //get isolated name of .desktop  (no path, no extension)
+  gchar *app = strrchr(appconf, '/') + 1;
+  int pos = strlen(app) - strlen(".desktop");
+  if (pos > 0 && !g_ascii_strcasecmp(app + pos, ".desktop")) {
+      app[pos]='\0';
+  }
+  //see if app is in the list of apps to show, regardless of onlyshowin flag
+  GSList *list = NULL;
+  for (list = ignore_list; list; list = g_slist_next(list)) {
+    if (!g_ascii_strcasecmp (app, (gchar *)list->data)) {
+        return TRUE;
+    }
+  }
+  return FALSE;
 }
 
 /////////////////////////////////////////////////////////
 // Read the system desktop entries and build a list of
 // applications that the Flash content should expose in
 // its interface.
-
-static GList *__parse_desktop_files(GList *list, 
-				    const char *directory,
-				    const char *group)
+static GList *__parse_desktop_files(plugin_context_t *c)
 {
-	DIR *dir_handle = NULL;
-	struct dirent *d = NULL;
-	struct stat buf;
-	gchar *current_path = NULL;
-	GKeyFile *key_file = NULL;
-	GError *error = NULL;
-	GtkIconTheme  *theme = gtk_icon_theme_get_default();
-	GtkIconInfo *info = NULL;
+        DIR *dir_handle = NULL;
+        struct dirent *d = NULL;
+        struct stat buf;
+        gchar *current_path = NULL;
+        GKeyFile *key_file = NULL;
+        GError *error = NULL;
+        GtkIconTheme *theme = gtk_icon_theme_get_default();
+        GtkIconInfo *info = NULL;
+	gchar **categories = NULL, **onlyshowin = NULL;
+	gint index = 0;
+	GSList *dir_list = NULL, *onlyshowin_ignore = NULL;
+	gboolean onlyshowin_filter = FALSE, show_app = FALSE;;
 
-	gtk_icon_theme_prepend_search_path(theme, ICON_DIR);
+	gtk_icon_theme_prepend_search_path(theme, PIXMAP_DIR);
 
-	if ((dir_handle = opendir(directory)) == NULL) {
-		g_warning("Error reading file '%s'\n", directory );
-		list;
+	// Setup two filters:
+	// onlyshowin_filter will not show apps for .desktop
+	// files that don't have OnlyShowIn=GNOME;Mobile
+	// onlyshowin_ignore will show an app regardless of onlyshowin
+	onlyshowin_filter = gconf_client_get_bool (c->client,
+                                                   ONLYSHOWIN_FILTER_KEY,
+                                                   NULL);
+
+	if (onlyshowin_filter) {
+		onlyshowin_ignore = gconf_client_get_list (c->client,
+                                                           ONLYSHOWIN_IGNORE_KEY,
+                                                           GCONF_VALUE_STRING,
+                                                           NULL);
 	}
 
-	while (d = readdir(dir_handle)) {
-		application_entry_t *i = NULL;
+        if ((dir_handle = opendir(DESKTOP_DIR)) == NULL) {
+                g_warning ("Error reading file '%s'\n", DESKTOP_DIR);
+        }
 
-		current_path = g_build_filename(directory, d->d_name, NULL);
+        while ((d = readdir(dir_handle)) != NULL) {
+                application_entry_t *i = NULL;
+		show_app = FALSE;
 
-		if ((stat(current_path, &buf) == 0) &&
-		    S_ISREG(buf.st_mode) &&
-		    g_str_has_suffix(current_path, DESKTOP_FILE_SUFFIX)) {
-			key_file = g_key_file_new();
+                current_path = g_build_filename(DESKTOP_DIR, d->d_name, NULL);
 
-			if ( g_key_file_load_from_file( key_file, current_path,
-					G_KEY_FILE_NONE, &error ) == FALSE ) {
+                if ((stat(current_path, &buf) == 0) &&
+                    S_ISREG(buf.st_mode) &&
+                    g_str_has_suffix(current_path, DESKTOP_FILE_SUFFIX)) {
+                        key_file = g_key_file_new();
 
-				g_warning("Error reading '%s': %s\n",
-						current_path, error->message);
-				g_error_free(error);
-				error = NULL;
-	                	g_free(current_path);
+                        if ( g_key_file_load_from_file( key_file, current_path,
+                                        G_KEY_FILE_NONE, &error ) == FALSE ) {
+
+                                g_warning("Error reading '%s': %s\n",
+                                                current_path, error->message);
+                                g_error_free(error);
+                                error = NULL;
+                                g_free(current_path);
+                                continue;
+                        }
+
+			//  If gconf key ONLYSHOWIN_FILTER_KEY == True, check for
+			//  OnlyShowIn=GNOME;Mobile in .desktop file
+			//  If not, skip app.  If no gconf key, show all apps regardless
+			if (onlyshowin_filter &&
+                            !is_app_in_ignore_list (onlyshowin_ignore, current_path)) {
+				onlyshowin = g_key_file_get_string_list (key_file,
+                                                                         DESKTOP_GROUP,
+                                                                         "OnlyShowIn",
+                                                                         NULL,
+                                                                         NULL);
+				if (onlyshowin) {
+					for (gint j = 0; onlyshowin[j]; j++) {
+						if (!g_ascii_strcasecmp (onlyshowin[j], "MOBILE")) {
+							show_app = TRUE;
+							break;
+						}
+					}
+					g_strfreev (onlyshowin);
+				}
+			} else {
+				show_app = TRUE;
+			}
+
+			if (!show_app) {
+				g_key_file_free (key_file);
+				g_free (current_path);
 				continue;
 			}
 
-			i = (application_entry_t *)
-				g_malloc0(sizeof(application_entry_t));
+                        i = (application_entry_t *)
+                                g_malloc0(sizeof(application_entry_t));
 
+			i->index = index++;
+                        i->name = get_localized_name(key_file, DESKTOP_GROUP);
+                        if (!i->name) {
+                                g_key_file_free(key_file);
+                                g_free(current_path);
+                                g_free(i);
+                                g_warning("Missing name entry");
+                                continue;
+                        }
+                        gchar *t = g_key_file_get_string(key_file,
+                                                         DESKTOP_GROUP,
+                                                         DESKTOP_ENTRY_ICON_FIELD,
+                                                         NULL);
+                        if (!t) {
+                                g_key_file_free(key_file);
+                                g_free(current_path);
+                                g_free(i->name);
+                                g_free(i);
+                                g_warning("Missing icon entry");
+                                continue;
+                        }
 
-			i->plugin = g_key_file_get_string(key_file,
-							  group,
-							  "X-control-panel-plugin",
-							  NULL);
-
-			i->name = get_localized_name(key_file, group);
-
-			if (!i->name) {
-				g_key_file_free(key_file);
-				g_free(current_path);
-				g_free(i);
-				g_warning("Missing name entry");
-				continue;
+			//if there is an extension, strip it.  simple logic will catch 90% (.jpg, .png)
+			int pos = strlen(t)-4;  //extension length = 4
+			if (pos>0 && t[pos]=='.') {
+				t[pos] = '\0';
 			}
 
-			gchar *t = g_key_file_get_string(key_file,
-							 group,
-							 DESKTOP_ENTRY_ICON_FIELD,
-							 NULL);
-			if (!t) {
-				g_key_file_free(key_file);
-				g_free(current_path);
-				g_free(i->name);
-				g_free(i);
-				g_warning("Missing icon entry");
-				continue;
+                        info = gtk_icon_theme_lookup_icon(theme, t, 48, GTK_ICON_LOOKUP_NO_SVG);
+			if (!info) {
+				//let's try the unknown icon
+				info = gtk_icon_theme_lookup_icon (theme, UNKNOWN_ICON_NAME, 
+                                                                   48, GTK_ICON_LOOKUP_NO_SVG);
+			}
+			if (info) {
+				i->icon = g_strdup (gtk_icon_info_get_filename (info));
+			} else {
+				i->icon = g_strdup ("null");
+			}
+                        g_free(t);
+
+                        i->service = g_key_file_get_string(key_file,
+                                                           DESKTOP_GROUP,
+                                                           DESKTOP_ENTRY_SERVICE_FIELD,
+                                                           NULL);
+                        i->exec = g_key_file_get_string(key_file,
+                                                        DESKTOP_GROUP,
+                                                        DESKTOP_ENTRY_EXEC_FIELD,
+                                                        NULL);
+                        if (!i->exec && !i->service) {
+                                g_key_file_free(key_file);
+                                g_free(current_path);
+                                g_free(i->name);
+                                g_free(i->icon);
+                                g_free(i);
+                                g_warning("Missing both exec and service");
+                                continue;
+                        }
+			i->cat = NULL;
+			categories = g_key_file_get_string_list(key_file,
+                                                                DESKTOP_GROUP,
+                                                                DESKTOP_ENTRY_CATEGORIES_FIELD,
+                                                                NULL,
+                                                                NULL);
+			
+			if (NULL != categories)
+			{
+				for (gint j = 0; categories[j]; j++) {
+					i->cat = g_hash_table_lookup (c->cat_table, categories[j]);
+					if (i->cat)
+						break;
+	                        }
+				g_strfreev (categories);
 			}
 
-			info = gtk_icon_theme_lookup_icon(theme, t, 48, GTK_ICON_LOOKUP_NO_SVG);
-			i->icon = g_strdup(gtk_icon_info_get_filename(info));
-
-			g_free(t);
-
-			i->service = g_key_file_get_string(key_file,
-							   group,
-							   DESKTOP_ENTRY_SERVICE_FIELD,
-							   NULL);
-			i->exec = g_key_file_get_string(key_file,
-							group,
-							DESKTOP_ENTRY_EXEC_FIELD,
-							NULL);
-			if (!i->exec && !i->service && !i->plugin) {
-				g_key_file_free(key_file);
-				g_free(current_path);
-				g_free(i->name);
-				g_free(i->icon);
-				g_free(i);
-				g_warning("Missing both exec and service");
-				continue;
-			}				
-
-
-			i->cat = g_key_file_get_string(key_file,
-						       group,
-						       DESKTOP_ENTRY_CATEGORIES_FIELD,
-						       NULL);
 			if (!i->cat)
 				i->cat = g_strdup ("Extras");
 
-			i->filename = g_strdup(d->d_name);
-			list = g_list_append(list, i);
-			g_key_file_free(key_file);
-		}
+			if (c->client) {
+				GSList *node;
+				dir_list = gconf_client_get_list (c->client,
+                                                                  CATEGORIES_KEY,
+                                                                  GCONF_VALUE_STRING,
+                                                                  NULL);
+				for (node = dir_list; node; node = node->next) {
+					if (!g_ascii_strcasecmp ((gchar *)node->data, i->cat))
+						break;
+				}
 
-		if (current_path)
-			g_free(current_path);
-	}
+				if (!node) {
+					dir_list = g_slist_append (dir_list, i->cat);
+					gconf_client_set_list (c->client,
+                                                               CATEGORIES_KEY,
+                                                               GCONF_VALUE_STRING,
+                                                               dir_list, NULL);
+				}
+			}
 
-	closedir(dir_handle);
-	g_free(d);
-	return list;
+                        i->filename = g_strdup(d->d_name);
+                        c->app_list = g_list_append(c->app_list, i);
+                        g_key_file_free(key_file);
+                }
+                if (current_path)
+                        g_free(current_path);
+        }
+
+        closedir(dir_handle);
+        g_free(d);
+        return c->app_list;
 }
 
-//TBD:  Get list from /usr/share/applications/<pkg>  MobileId
-static void build_app_list(plugin_context_t *c)
+static GHashTable *parse_menus (plugin_context_t *c, xmlNode *list)
 {
-	c->app_list = __parse_desktop_files(c->app_list, 
-					    CP_PLUGINS_DIR, 
-					    DESKTOP_GROUP);
-	c->app_list = __parse_desktop_files(c->app_list, 
-					    DESKTOP_DIR,
-					    DESKTOP_GROUP);
-	
-	c->app_list = g_list_sort(c->app_list, compare_items);
-	for (int i=0; i<g_list_length(c->app_list); i++) {
-		GList *l = g_list_nth(c->app_list, i);
-		application_entry_t *item = (application_entry_t *)l->data;
-		item->index = i;
-	}
-	
-	return;
+  xmlNode *iter, *iterchild;
+  gchar *name = NULL, *category = NULL;
+
+  for (iter = list->children; iter != NULL; iter = iter->next )
+  {
+    if (!strcmp ((gchar *)iter->name, "Menu"))
+      c->cat_table = parse_menus (c, iter);
+    else if (!strcmp ((gchar *)iter->name, "Name"))
+      name = g_strdup ((gchar *)iter->last->content);
+    else if (!strcmp ((gchar *)iter->name, "Include"))
+    {
+      for (iterchild = ((iter->children)->next)->children; iterchild != NULL; iterchild = iterchild->next)
+      {
+        if (strcmp ((gchar *)iterchild->name, "Category"))
+          continue;
+        category = g_strdup ((gchar *)iterchild->last->content);
+      }
+    }
+  }
+
+  if (name && category)
+    g_hash_table_replace (c->cat_table, category, name);
+
+  return c->cat_table;
 }
 
+static GHashTable *set_up_cat_table(plugin_context_t *c, const gchar *menu_path)
+{
+  xmlDoc *doc = NULL;
+  xmlNode *root = NULL;
+
+  doc = xmlParseFile (menu_path);
+  root = xmlDocGetRootElement (doc);
+
+  if (NULL == root)
+    return NULL;
+
+  return parse_menus (c, root);
+}
 
 //gconf notification callback (registered in init below)
 static void background_changed (GConfClient *client, guint id,
@@ -870,22 +993,8 @@ static void active_cat_changed (GConfClient *client, guint id,
 	}
 }
 
-static void window_added_cb(HDWM *hdwm, HDWMEntryInfo *entry_info, gpointer data)
-{
-   	DBG_LOG ("window_added_cb");
-	hide_banner();
-}
- 
-static void window_stack_change_cb(HDWM *hdwm, HDWMEntryInfo *entry_info, gpointer data)
-{
-	DBG_LOG ("window_stack_change_cb");
-	hide_banner();
-}
-
 static void mobile_basic_home_plugin_init (MobileBasicHomePlugin *home_plugin)
 {
-	DBG_LOG ("init");
-
 	plugin_context_t *context = (plugin_context_t *)g_malloc0(
 		sizeof(plugin_context_t));
 
@@ -912,20 +1021,22 @@ static void mobile_basic_home_plugin_init (MobileBasicHomePlugin *home_plugin)
 						  ACTIVE_CAT_KEY,
 						  NULL);
 
-	//subscribe for window change so we can hide banner after app startup
-        g_signal_connect (hd_wm_get_singleton (),
-                          "entry_info_added",
-                          G_CALLBACK(window_added_cb),
-                          (gpointer) context);
+        //initialize category list key
+        gconf_client_set_list (context->client,
+                               CATEGORIES_KEY,
+                               GCONF_VALUE_STRING,
+                               NULL, NULL);
 
-        g_signal_connect (hd_wm_get_singleton (),
-                          "entry_info_stack_changed",
-                          G_CALLBACK(window_stack_change_cb),
-                          (gpointer) context);
+        //use libwnck to dismiss banner when window or dialog appear
+        init_window_helper (context);
+
+	//set up categories table
+	context->cat_table = g_hash_table_new (g_str_hash, g_str_equal);
+	context->cat_table = set_up_cat_table (context, APPLICATIONS_MENU);
+	context->cat_table = set_up_cat_table (context, PREFERENCES_MENU);
 
 	//create application list
-	context->app_list = NULL;
-	build_app_list(context);
+	context->app_list = __parse_desktop_files(context);
 
 	gtk_moz_embed_set_profile_path(PROFILE_DIR, PROFILE_NAME);
   
@@ -990,8 +1101,6 @@ static void mobile_basic_home_plugin_init (MobileBasicHomePlugin *home_plugin)
 		g_free(url);
 	}
 
-	DBG_LOG ("loaded main URL");
-
 	gtk_widget_show_all(context->container);
 	gtk_container_add (GTK_CONTAINER (home_plugin), context->container);
 	gtk_signal_connect(GTK_OBJECT(context->container), "js_status",
@@ -1003,7 +1112,7 @@ static void mobile_basic_home_plugin_init (MobileBasicHomePlugin *home_plugin)
 	gconf_client_add_dir (context->client, BKGD_PATH,
 			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	gconf_client_notify_add (context->client,
-				 BKGD_FILE,
+				 BKGD_PATH,
 				 (GConfClientNotifyFunc) background_changed,
 				 context, NULL, NULL);
 
@@ -1021,8 +1130,6 @@ static void mobile_basic_home_plugin_init (MobileBasicHomePlugin *home_plugin)
 				 MARQUEE_GCONF_PATH,
 				 (GConfClientNotifyFunc) active_cat_changed,
 				 context, NULL, NULL);
-
-	DBG_LOG ("init finished"); 
 }
 
 static GdkFilterReturn mobile_basic_home_x_event_handler (GdkXEvent *xevent, GdkEvent *event, gpointer data)
@@ -1080,11 +1187,5 @@ mobile_basic_home_plugin_class_init (MobileBasicHomePluginClass *klass)
 {
   HildonDesktopHomeItemClass *item_class = HILDON_DESKTOP_HOME_ITEM_CLASS (klass);
 
-  //Commented out until we get change in hildon-desktop into hardy PPA
-  //item_class->screen_changed = home_screen_changed;
+  item_class->screen_changed = home_screen_changed;
 }
-
-
-
-
-
