@@ -11,6 +11,7 @@
  * Contributors:
  * Michael Frey <michael.frey@pepper.com>
  * Rusty Lynch <rusty.lynch@intel.com>
+ * Bill Filler <bill.filler@canonical.com> 
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -33,6 +34,8 @@
 
 #include <glib-object.h>
 #include <libhildondesktop/hildon-desktop-home-item.h>
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE
+#include <libwnck/libwnck.h>
 
 G_BEGIN_DECLS
 
